/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/

#include <string.h> 
#include <libadminutil/srvutil.h>
#include "admutil_pvt.h"

#ifdef XP_WIN32
#define strcasecmp stricmp
#define strncasecmp _strnicmp
#endif
 
PR_IMPLEMENT(AttrNameList)
getServerDNList(AdmldapInfo info)
{
  char        *domainDN = NULL, *isie = NULL, *sie = NULL;
  char        *ptr = NULL, *adminName = NULL;
  PsetHndl    domainPset;
  int         errorCode;
  ListNodePtr dnList=NULL, node=NULL;
  AttrNameList  nl;

  isie = admldapGetISIEDN(info);
  /*
   * ISIE sample:
   * isie: cn=<NAME> Administration Server, cn=Server Group, cn=myhost.example.com, ou=example.com, o=NetscapeRoot
   */
  ptr = strchr(isie, ',');
  if (!ptr) goto err;

  *ptr = '\0'; /* to cut out "<NAME> Administration Server" later */

  sie = admldapGetSIEDN(info);

  ptr++;

  while (*ptr == ' ' &&  *ptr != '\0') ptr++;

  if (*ptr == '\0') goto err;

  domainDN = PL_strdup(ptr);
  if (!domainDN) goto err;

  adminName = strchr(isie, '=');
  if (!adminName) goto err;
  adminName++;

  /* Use domainDN to create a pset */
  domainPset = psetRealCreate(admldapGetHost(info),
			      admldapGetPort(info),
			      domainDN,
			      sie,
			      admldapGetSIEPWD(info),
			      NULL,
			      &errorCode);

  if (!domainPset) goto err;

  nl = retrieveSIEs(domainPset, domainDN, adminName);

  psetDelete(domainPset);  /* free sie, domainDN, internally */
  if (isie) PR_Free(isie);
  return nl;

err:
  if (isie) PR_Free(isie);
  if (sie) PR_Free(sie);
  if (domainDN) PR_Free(domainDN);
  return NULL;
}

PR_IMPLEMENT(AttrNameList)
retrieveSIEs(PsetHndl domainPset, char* domainDN, char *adminName)
{
  char          *sieDN = NULL, *name, *sieName, *attrName;
  int           errorCode, i, isieLen;
  ListNodePtr   dnList=NULL, node=NULL;
  AttrNameList  nl, nl1, sieList, childrenList;

  childrenList = psetGetChildren(domainPset, "", &errorCode);
  if (errorCode) return NULL;

  nl = childrenList;
  while (name = *nl++) {
    if (strcasecmp(name, adminName)) {
      sieList = psetGetChildren(domainPset, name, &errorCode);
      nl1 = sieList;
      isieLen = PL_strlen(name);
      while (sieName = *nl1++) {
	attrName = sieName+isieLen+1;
	if (strcasecmp(attrName, "tasks")) {
	  sieDN = attrName2dn(sieName, domainDN);
	  node = createListNode(sieDN, NULL, 0);
	  if (dnList) listCat(dnList, node);
	  else dnList = node;
	  PR_Free(sieDN);
	  sieDN = NULL;
	}
      }
      if (sieList) deleteAttrNameList(sieList);
    }
  }
  deleteAttrNameList(childrenList);
  /* Construct the returned name list  */
  if (!dnList) return NULL;
  nl = createAttrNameList(listCount(dnList));
  i=0;
  node = dnList;
  while(node) {
    addName(nl, i++, node->name);
    node=node->next;
  }

  listDestroy(dnList);
  return nl;
}

  
PR_IMPLEMENT(AttributeList)
getInstalledServerDNList(AdmldapInfo info)
{
  char          *domainDN, *isie;
  PsetHndl      domainPset;
  AttributeList resultList;
  int           errorCode;

  isie = admldapGetISIEDN(info);

  domainDN=strchr(isie, ',');
  if (!domainDN) return NULL;

  domainDN++;

  while (*domainDN == ' ' &&  *domainDN != '\0') domainDN++;

  if (*domainDN == '\0') return NULL;

  /* Use domainDN to create a pset */

  domainPset = psetRealCreate(admldapGetHost(info),
			      admldapGetPort(info),
			      domainDN,
			      admldapGetSIEDN(info),
			      admldapGetSIEPWD(info),
			      NULL,
			      &errorCode);

  if (!domainPset) return NULL;

  resultList = retrieveISIEs(domainPset, domainDN);
  psetDelete(domainPset);

  return resultList;

}

PR_IMPLEMENT(AttributeList)
retrieveISIEs(PsetHndl domainPset, char* domainDN)
{
  char          *name;
  int           errorCode, i, cnt = 0;
  ListNodePtr   dnList=NULL, node=NULL;
  char          dn[1024], attrName[256];
  AttributeList resultList = NULL;
  AttrNameList  childrenList, nl;
  char          *nickName;

  childrenList = psetGetChildren(domainPset, "", &errorCode);
  if (errorCode) return NULL;

  nl = childrenList;
  while (name = *nl++) {
    PR_snprintf (attrName, sizeof(attrName), "%s.nsNickName", name);
    nickName = psetGetAttrSingleValue(domainPset, attrName, &errorCode);
    if (nickName) {
      PR_snprintf (dn, sizeof(dn), "cn=%s, %s", name, domainDN);
      node = createListNode(nickName, PL_strdup(dn), 1);
      if (dnList)  listCat(dnList, node);
      else dnList = node;
      PR_Free(nickName);
    }
  }
  deleteAttrNameList(childrenList);

  cnt = listCount(dnList);
  node = dnList;
  i = 0;
  if (cnt > 0) {
    resultList = createAttributeList(cnt);
    while (node) {
      addSingleValueAttribute(resultList, 
			      i++, 
			      node->name,
			      (char*)(node->val));
      node = node->next;
    }
  }

  listDestroy(dnList);

  return resultList;
}
