/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
#include <string.h>
#include "libadminutil/admutil.h"
#include "libadmsslutil/psetcssl.h"
#include "libadmsslutil/srvutilssl.h"


PR_IMPLEMENT(AttrNameList)
getServerDNListSSL(AdmldapInfo info)
{
  char        *domainDN = NULL, *isie = NULL, *sie = NULL;
  char        *ptr = NULL, *adminName = NULL;
  PsetHndl    domainPset;
  int         errorCode;
  AttrNameList  nl;

  isie = admldapGetISIEDN(info);
  /*
   * ISIE sample:
   * isie: cn=<NAME> Administration Server, cn=Server Group, cn=myhost.example.com, ou=example.com, o=NetscapeRoot
   */

  ptr=strchr(isie, ',');
  if (!ptr) goto err;

  *ptr = '\0'; /* to cut out "<NAME> Administration Server" later */

  sie = admldapGetSIEDN(info);

  ptr++;

  while (*ptr == ' ' &&  *ptr != '\0') ptr++;

  if (*ptr == '\0') goto err;

  domainDN = PL_strdup(ptr);
  if (!domainDN) goto err;

  adminName = strchr(isie, '=');
  if (!adminName) goto err;
  adminName++;

  /* Use domainDN to create a pset */
  domainPset = psetRealCreateSSL(admldapGetHost(info),
                                 admldapGetPort(info),
                                 admldapGetSecurity(info),
                                 domainDN,
                                 sie,
                                 admldapGetSIEPWD(info),
                                 NULL,
                                 &errorCode);

  if (!domainPset) goto err;

  nl = retrieveSIEs(domainPset, domainDN, adminName);

  psetDelete(domainPset); /* free sie, domainDN, internally */
  if (isie) PR_Free(isie);
  return nl;

err:
  if (isie) PR_Free(isie);
  if (sie) PR_Free(sie);
  if (domainDN) PR_Free(domainDN);
  return NULL;
}
  
PR_IMPLEMENT(AttributeList)
getInstalledServerDNListSSL(AdmldapInfo info)
{
  char          *domainDN, *isie;
  PsetHndl      domainPset;
  int           errorCode;
  AttributeList resultList;

  isie = admldapGetISIEDN(info);

  domainDN=strchr(isie, ',');
  if (!domainDN) return NULL;

  domainDN++;

  while (*domainDN == ' ' &&  *domainDN != '\0') domainDN++;

  if (*domainDN == '\0') return NULL;

  /* Use domainDN to create a pset */
  domainPset = psetRealCreateSSL(admldapGetHost(info),
                                 admldapGetPort(info),
                                 admldapGetSecurity(info),
                                 domainDN,
                                 admldapGetSIEDN(info),
                                 admldapGetSIEPWD(info),
                                 NULL,
                                 &errorCode);

  if (!domainPset) return NULL;
  resultList = retrieveISIEs(domainPset, domainDN);
  psetDelete(domainPset);

  return resultList;

}

